@@ -43,12 +43,12 @@
 #include <syslog.h>
 
 #ifndef VERSION
-#define VERSION "1.4"
+#define VERSION "1.5"
 #endif
 
 static char *self = NULL;
 
-static size_t bind_service (const char *service, int family, int* socks, size_t max_socks)
+static size_t bind_service (const char *listen_addr, const char *service, int family, int* socks, size_t max_socks)
 {
   struct addrinfo *addr, *it;
   struct addrinfo hints;
@@ -60,7 +60,7 @@ static size_t bind_service (const char *service, int family, int* socks, size_t
   hints.ai_family = family;
   hints.ai_socktype = SOCK_STREAM;
 
-  err = getaddrinfo(NULL, service, &hints, &addr);
+  err = getaddrinfo(listen_addr, service, &hints, &addr);
   if (err != 0) {
     fprintf(stderr, "%s: bind_service(%s): %s\n", self, service, gai_strerror(err));
     return count;
@@ -219,6 +219,9 @@ static void show_help ()
     "   -s<file>       read secrets from file (required)\n"
     "   -d<domain>     set domain name for rewrite (required)\n"
     "   -a<char>       set first separator character which can be one of: -=+ (default: =)\n"
+    "   -n<num>        length of hash to be used in rewritten addresses (default: 4)\n"
+    "   -N<num>        minimum length of hash to accept for validation (default: 4)\n"
+    "   -l<addr>       set socket listen address (default: 127.0.0.1)\n"
     "   -f<port>       set port for the forward SRS lookup (default: 10001)\n"
     "   -r<port>       set port for the reverse SRS lookup (default: 10002)\n"
     "   -p<pidfile>    write process ID to pidfile (default: none)\n"
@@ -241,9 +244,9 @@ typedef void(*handle_t)(srs_t*, FILE*, const char*, const char*, const char**);
 
 int main (int argc, char **argv)
 {
-  int opt, timeout = 1800, family = AF_UNSPEC;
+  int opt, timeout = 1800, family = AF_UNSPEC, hashlength = 0, hashmin = 0;
   int daemonize = FALSE;
-  char *forward_service = NULL, *reverse_service = NULL,
+  char *listen_addr = NULL, *forward_service = NULL, *reverse_service = NULL,
        *user = NULL, *domain = NULL, *chroot_dir = NULL;
   char separator = '=';
   char *secret_file = NULL, *pid_file = NULL;
@@ -259,12 +262,13 @@ int main (int argc, char **argv)
   size_t socket_count = 0, sc;
   int sockets[4] = { -1, -1, -1, -1 };
   handle_t handler[4] = { 0, 0, 0, 0 };
+  int fd, maxfd;
 
   excludes = (const char**)calloc(1, sizeof(char*));
   tmp = strrchr(argv[0], '/');
   if (tmp) self = strdup(tmp + 1); else self = strdup(argv[0]);
 
-  while ((opt = getopt(argc, argv, "46d:a:f:r:s:u:t:p:c:X::Dhev")) != -1) {
+  while ((opt = getopt(argc, argv, "46d:a:l:f:r:s:n:N:u:t:p:c:X::Dhev")) != -1) {
     switch (opt) {
       case '?':
         return EXIT_FAILURE;
@@ -280,6 +284,9 @@ int main (int argc, char **argv)
       case 'a':
         separator = *optarg;
         break;
+      case 'l':
+        listen_addr = strdup(optarg);
+        break;
       case 'f':
         forward_service = strdup(optarg);
         break;
@@ -292,6 +299,12 @@ int main (int argc, char **argv)
       case 's':
         secret_file = strdup(optarg);
         break;
+      case 'n':
+        hashlength = atoi(optarg);
+        break;
+      case 'N':
+        hashmin = atoi(optarg);
+        break;
       case 'p':
         pid_file = strdup(optarg);
         break;
@@ -330,6 +343,10 @@ int main (int argc, char **argv)
           domain = strdup(getenv("SRS_DOMAIN"));
         if ( getenv("SRS_SEPARATOR") != NULL )
           separator = *getenv("SRS_SEPARATOR");
+        if ( getenv("SRS_HASHLENGTH") != NULL )
+          hashlength = atoi(getenv("SRS_HASHLENGTH"));
+        if ( getenv("SRS_HASHMIN") != NULL )
+          hashmin = atoi(getenv("SRS_HASHMIN"));
         if ( getenv("SRS_FORWARD_PORT") != NULL )
           forward_service = strdup(getenv("SRS_FORWARD_PORT"));
         if ( getenv("SRS_REVERSE_PORT") != NULL )
@@ -382,6 +399,11 @@ int main (int argc, char **argv)
   if (forward_service == NULL) forward_service = strdup("10001");
   if (reverse_service == NULL) reverse_service = strdup("10002");
 
+  /* Close all file descriptors (std ones will be closed later). */
+  maxfd = sysconf(_SC_OPEN_MAX);
+  for(fd = 3; fd < maxfd; fd++)
+    close(fd);
+
   /* The stuff we do first may not be possible from within chroot or without privileges */
 
   /* Open pid file for writing (the actual process ID is filled in later) */
@@ -404,11 +426,11 @@ int main (int argc, char **argv)
     return EXIT_FAILURE;
   }
   /* Bind ports. May require privileges if the config specifies ports below 1024 */
-  sc = bind_service(forward_service, family, &sockets[socket_count], 4 - socket_count);
+  sc = bind_service(listen_addr, forward_service, family, &sockets[socket_count], 4 - socket_count);
   if (sc == 0) return EXIT_FAILURE;
   while (sc-- > 0) handler[socket_count++] = handle_forward;
   free (forward_service);
-  sc = bind_service(reverse_service, family, &sockets[socket_count], 4 - socket_count);
+  sc = bind_service(listen_addr, reverse_service, family, &sockets[socket_count], 4 - socket_count);
   if (sc == 0) return EXIT_FAILURE;
   while (sc-- > 0) handler[socket_count++] = handle_reverse;
   free (reverse_service);
@@ -469,6 +491,10 @@ int main (int argc, char **argv)
   fclose (sf);
 
   srs_set_separator (srs, separator);
+  if (hashlength)
+    srs_set_hashlength (srs, hashlength);
+  if (hashmin)
+    srs_set_hashmin (srs, hashmin);
 
   for (sc = 0; sc < socket_count; ++sc) {
     fds[sc].fd = sockets[sc];
